#!/bin/bash

#yaml Parser





Parse_yaml() {
   local prefix=$2
   local s='[[:space:]]*' w='[a-zA-Z0-9_]*' fs=$(echo @|tr @ '\034')
   sed -ne "s|^\($s\):|\1|" \
        -e "s|^\($s\)\($w\)$s:$s[\"']\(.*\)[\"']$s\$|\1$fs\2$fs\3|p" \
        -e "s|^\($s\)\($w\)$s:$s\(.*\)$s\$|\1$fs\2$fs\3|p"  $1 |
   awk -F$fs '{
      indent = length($1)/2;
      vname[indent] = $2;
      for (i in vname) {if (i > indent) {delete vname[i]}}
      if (length($3) > 0) {
         vn=""; for (i=0; i<indent; i++) {vn=(vn)(vname[i])("_")}
         printf("%s%s%s=\"%s\"\n", "'$prefix'",vn, $2, $3);
      }
   }'
}

# Checks if the paths given are correct.

Test_arguments()
{
  # Nemo path correct?
  if ! test -d "${Nemo_path}"; then
      echo "Nemo relative path: ${Nemo_path} is not found"
      echo
      exit 1
  fi

  # cfg exists?
    if ! test -d "${Nemo_path}/cfgs/${cfg}"; then
      echo "configuration: ${cfg} doesn't exists in ${Nemo_path}/cfgs dir"
      echo
      exit 1
    fi
  

   # arch exists?
  if ! test -f "${Nemo_path}/arch/arch-${arch}.fcm"; then
      echo "architecture: arch-${arch}.fcm doesn't exists in ${Nemo_path}/arch dir"
      echo
      exit 1
  fi

  # scheduler correct?
  if [ $Jobs_scheduler != "slurm" ] && [ $Jobs_scheduler != "lsf" ]; then
      echo "$Jobs_scheduler is not a valid scheduler"
      echo
      exit 1
  fi

  # $EXTRAE_HOME correct ?
  if ! test -d "${EXTRAE_HOME}"; then
    echo "Nemo relative path: ${EXTRAE_HOME} is not found"
    echo
    exit 1

  else

    export EXTRAE_HOME=$EXTRAE_HOME
    sed -i 's|home=.*|home="'"$EXTRAE_HOME"'"|g' extrae.xml

  fi

  # Adding -d to Nemo makefile if variable not empty
  if [ ! -z "$comp_cfg" ]; then
    comp_cfg="-d $comp_cfg"
  fi
  
  # Creating auxiliar vars for submiting jobs
  if [ $Jobs_scheduler == "slurm" ];then 

    job="sbatch"

  else

    job="bsub"

  fi

  module load $Modules
}


# Checks if compiler flag -g is present in the arch file, this flag is needed for the correct creation of traces.



Test_Comp()
{
  line=$(sed -n '/^%FCFLAGS /p' $Nemo_path/arch/arch-$arch.fcm)
  if ! echo "${line}"|grep -q "\-g\b" || [ $compile == "true" ]; then

    if [ $compile == "true" ]; then
      echo '$compile is assigned true compiling Nemo'
    else
      echo "-g flag not found in arch-${arch}.fcm "
    fi

    echo
    echo "Creating new arch file to compile with extrae arch-${arch}_EXTRAE.fcm"
    echo

    
    cp ${Nemo_path}/arch/"arch-${arch}".fcm ${Nemo_path}/arch/arch-${arch}_EXTRAE.fcm
    sed -i '/%FCFLAGS/ s/$/ -g --finstrument-functions/' ${Nemo_path}/arch/arch-${arch}_EXTRAE.fcm

    python3 $dirJob_Creator.py -f "compile" -j "compile" --set-core ${Jobs_n_cores} -s $Jobs_scheduler --set-time $time --set-queue $queue -w " cd ${Nemo_Path}
    ./makenemo -r ${cfg} -n ${name_cfg} -m ${arch}_EXTRAE -j$Jobs_n_cores $comp_cfg" 

  
    echo 'compiling with -g flag'
    echo
    
    $job --wait compile.$Jobs_scheduler

    cd $dir
    cp ${Nemo_path}/cfgs/${name_cfg}/EXP00/* .

    

    if [[ $comp_cfg == "-d OCE del_key 'key_si3 key_top'" ]]; then

      sed -i '/_def_nemo-ice.xml\|def_nemo-pisces.xml/d' context_nemo.xml #DELETE ICE AND PISCES CONTEXT (NOT USED)

    fi

    #Solving NEMO 4.2 Errors
    #Remove this line if the Nemo version is not 4.2. 
    sed -i 's|ln_zdfiwm * = .true.|ln_zdfiwm   = .false.|g' namelist_cfg #CHANGE DUE TO NON EXISTING FILES
    
    
    
  fi
}

Create_metrics()
{


  #Changing iterations, big traces generate problems.
    sed -i 's|nn_itend * =.*|nn_itend    =       16  !  last  time step (std 5475)|g' namelist_cfg

  # Run nemo with extrae

  echo "Creating trace ..."
  python3 Job_Creator.py -f "run" -j "run" --set-core ${Jobs_n_cores} -s $Jobs_scheduler --set-time $time --set-queue $queue -w "mpirun -np ${Jobs_n_cores} ./trace.sh ./nemo" 

  $job --wait run.$Jobs_scheduler

  # Cut trace to less than 1000 MiB 

  paramedir -c nemo.prv cutter_example.xml -o trace.prv
  echo "Trace information stored in trace.prv , trace.pcf and trace.row "
  echo "Creating metrics and storing theme in Metrics folder"

  # Creating folder
  if ! test -d "Metrics"; then
       mkdir Metrics
  fi
 
  cp trace.prv Metrics/
  cp trace.pcf Metrics/
  cp trace.row Metrics/
  python3 Job_Creator.py -f "modelfactors" -j "modelfactors" --set-core ${Jobs_n_cores} -s $Jobs_scheduler --set-time $time --set-queue $queue -w "modelfactors.py trace.prv" 
  cp modelfactors.$Jobs_scheduler Metrics/
  cd Metrics

  $job --wait modelfactors.$Jobs_scheduler
  
}



#Loads needed: intel/2019.4 gcc impi/2017.4 netcdf/4.4.1.1 hdf5/1.8.19 EXTRAE/3.7.1 perl Paraver Dinemas


if [ $# -gt 0 ]; then
  echo "This script does not accept arguments, parameters need to be added to config.yaml aborting"
  exit 1
fi

#Default Init
echo
echo "Using the following configuration:"
echo


Parse_yaml config.yaml
yaml_string="$(Parse_yaml config.yaml)"
eval "${yaml_string}"
echo

#Var init

dir=$(pwd)
Nemo_path="${Nemo_path:-"."}"
Jobs_n_cores="${Jobs_n_cores:-48}"
Jobs_scheduler="${Jobs_scheduler:-"slurm"}"
time="${Jobs_time:-"0"}"
queue="${Jobs_queue:-""}"
compile="${Compilation_compile:-"false"}"
cfg="${Compilation_ref:-"ORCA2_ICE_PISCES"}"
arch="${Compilation_arch:-"X64_MN4"}"
name_cfg="${Compilation_name:-"ORCA2_EXTRAE"}"
comp_cfg="${Compilation_sub:-""}"
Modules="${Modules:-""}"
EXTRAE_HOME="${EXTRAE_HOME:-""}"

Test_arguments
Test_Comp
Create_metrics
